﻿using C1.Chart;
using C1.Win.Chart;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using C1.Win.Chart;
using C1.Win.C1FlexGrid;

namespace BreakEvenAnalysis
{
    public partial class BreakEvenAnalysisOutput : Form
    {
        int unit;
        private Font font;
        List<BreakEvenPoint> breakEvenList = new List<BreakEvenPoint>();

        private void c1Button1_Click(object sender, EventArgs e)
        {
            for (int i = 1; i < Application.OpenForms.Count; i++)
            {
                Application.OpenForms[i].Close();
            }
            this.Close();
            //Reset the values of the Input Form
            C1.Win.C1Input.C1NumericEdit numberEdit = Application.OpenForms[0].Controls[0] as C1.Win.C1Input.C1NumericEdit;
            numberEdit.Value = 0;
            C1.Win.C1Input.C1TextBox text = Application.OpenForms[0].Controls[1] as C1.Win.C1Input.C1TextBox;
            text.Value = null;
            text = Application.OpenForms[0].Controls[2] as C1.Win.C1Input.C1TextBox;
            text.Value = null;
            text = Application.OpenForms[0].Controls[3] as C1.Win.C1Input.C1TextBox;
            text.Value = null;
        }

        public BreakEvenAnalysisOutput(double fixedCost, double variableCost, double unitSellingPrice, int unit)
        {
            InitializeComponent();
            font = new System.Drawing.Font("Lucida Bright", 15F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.c1Label1.Font = font;
            this.c1Label2.Font = font;
            this.BackColor = Color.Black;
            this.tableLayoutPanel1.BackColor = Color.Black;
            this.tableLayoutPanel2.BackColor = Color.Black;
            this.panel1.BackColor = Color.Black;
            this.panel2.BackColor = Color.Black;
            this.panel3.BackColor = Color.Black;
            this.flexChart1.BackColor = Color.Black;
            this.c1FlexGrid1.BackColor = Color.Black;
            //Assign the StarWidth so that every column is visible
            for (int i = 1; i < c1FlexGrid1.Cols.Count; i++)
                c1FlexGrid1.Cols[i].StarWidth = "*";
            this.unit = unit;
            c1Label1.Text = "Break Even Point : ";
            //Calculate Break Even Point via BreakEvenPoint Formula
            double value = (int)(fixedCost / (unitSellingPrice - variableCost));
            c1Label2.Text = value.ToString();
            //Clear Existing Series
            flexChart1.Series.Clear();
            //Add a Break Even Chart into FlexChart to show the Break Even Analysis
            flexChart1.Series.Add(new BreakEven() { SalesPrice = unitSellingPrice, FixedCost = fixedCost, VariableCost = variableCost });
            flexChart1.Legend.Position = Position.Right;
            BreakEven breakEvenChart = flexChart1.Series[0] as BreakEven;
            flexChart1.AxisX.Title = "Number Of Units";
            flexChart1.AxisY.Title = "Fixed Cost";
            int units = 0, sales = 0, totalCost = (int)fixedCost, iteration = 0;
            //Calculate data for particular interval of units and add it into the list
            breakEvenList.Add(new BreakEvenPoint(units, sales, totalCost));

            while ((sales - totalCost) <= 0)
            {
                units = units + unit;
                sales = (int)GetSalesRevenue(breakEvenChart, units);
                totalCost = (int)GetTotalCost(breakEvenChart, units);
                iteration = iteration + 1;
                breakEvenList.Add(new BreakEvenPoint(units, sales, totalCost));
            }

            while (iteration > 0)
            {
                units = units + unit;
                sales = (int)GetSalesRevenue(breakEvenChart, units);
                totalCost = (int)GetTotalCost(breakEvenChart, units);
                iteration = iteration - 1;
                breakEvenList.Add(new BreakEvenPoint(units, sales, totalCost));
            }
            //Assign the interval of unit to X-Axis for which we want to see the record 
            flexChart1.AxisX.MinorUnit = unit;
            flexChart1.AxisX.MajorUnit = unit;
            //Assign the list of data as the datasource of the grid
            c1FlexGrid1.DataSource = breakEvenList;
            this.Load += Form2_Load;
        }

        private void Form2_Load(object sender, EventArgs e)
        {
            c1FlexGrid1.AllowEditing = false;
            //Add a new Column to the Grid to show the Profit/Loss after selling particular units 
            Column columnProfitLoss = c1FlexGrid1.Cols.Add();
            columnProfitLoss.Name = columnProfitLoss.Caption = "Profit/Loss";
            columnProfitLoss.AllowExpressionEditing = true;
            //Add Expression to calculate the values based on existing columns' data
            columnProfitLoss.Expression = "[Sales] - [TotalCost]";
            //Add CellStyle to show the loss
            CellStyle cs = c1FlexGrid1.Styles.Add("Loss");
            cs.ForeColor = Color.FromArgb(255, 51, 51);
            //Add CellStyle to show the Profit
            CellStyle cs1 = c1FlexGrid1.Styles.Add("Profit");
            cs1.ForeColor = Color.FromArgb(102, 204, 0);
            //Assing the CellStyle to particular cell based on its value
            for (int i = 1; i < c1FlexGrid1.Rows.Count; i++)
            {
                if (Convert.ToInt32(c1FlexGrid1[i, 4]) > 0)
                {
                    c1FlexGrid1.SetCellStyle(i, 4, cs1);
                }
                else
                {
                    c1FlexGrid1.SetCellStyle(i, 4, cs);
                }
            }
        }

        //Method to Get the Total Cost at particular unit
        double GetTotalCost(BreakEven breakEven, int nUnits)
        {
            return breakEven.FixedCost + breakEven.VariableCost * nUnits;
        }

        //Method to Get the Sales Revenue at particular unit
        double GetSalesRevenue(BreakEven breakEven, int nUnits)
        {
            return breakEven.SalesPrice * nUnits;
        }

    
    }

    public class BreakEvenPoint
    {
        int unitSold, sales, totalCost;
        public BreakEvenPoint(int unitSold, int sales, int totalCost)
        {
            this.unitSold = unitSold;
            this.sales = sales;
            this.totalCost = totalCost;
        }
        public int UnitSold
        {
            get
            { return unitSold; }
            set
            { unitSold = value; }
        }
        public int Sales
        {
            get
            { return sales; }
            set
            { sales = value; }
        }
        public int TotalCost
        {
            get
            { return totalCost; }
            set
            { totalCost = value; }
        }
    }
}
